/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbstractRulePackageVO.as 4876 2009-08-11 14:22:08Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage
{

import flash.events.Event;

import mx.collections.ArrayCollection;

import org.ow2.jasmine.monitoring.eos.rules.common.utils.RulePackageNameFormatter;

/**
 * The base implementation of any kind of rule package
 *
 * @author Julien Nicoulaud
 */
[RemoteClass(alias="org.ow2.jasmine.rules.manager.model.AbstractRulePackage")]
public class AbstractRulePackageVO extends AbstractRulePackageMemberVO
                                   implements IRulePackageVO
{
	
	// =========================================================================
	// Properties
	// =========================================================================
	
	// Data values
	
    /**
    * The unique system name of the rule package. Identifies the object, must be
    * different for each rule package. Automatically generated from the rule
    * package "display" name.
    * @example "org.ow2.jasmine.monitoring.rules.example.ExamplePackage"
    */
    private var _systemName : String;
    
    /**
    * The previous systemName of the package in the case where it 
    * was modified. Used to identify the old rule package.
    */
    private var _previousSystemName : String = null;
    
    /**
    * The displayed name of the rule package. This is the name that is choosed by
    * the user. Used to generate automatically the systemName property.
    * @example "Example Package"
    */
    private var _displayName : String;
    
    /**
     * The name of the rule engine instance on which the rule is executed.
     */
    private var _ruleEngineInstanceName : String;
    
	/**
	* Allowed states
	*/
	public static var DEACTIVATED : String = "deactivated";
	public static var ACTIVATED : String = "activated";
	public static var ACTIVATION_REQUESTED : String = "activationRequested";
    public static var DEACTIVATION_REQUESTED : String = "deactivationRequested";
	
    /**
    * The state of the rule package. Values can be:
    *   - Activated
    *   - Decativated
    *   - Activation requested
    *   - Deactivation requested
    */
    private var _state : String = DEACTIVATED;
    	
	/**
	* The description of the rule package
    */
	private var _description : String;
	
	
	// Transient properties
	
    /**
    * A name formatter displayName => systemName
    */
    [Transient]
    private var rulepackageNameFormatter : RulePackageNameFormatter
         = new RulePackageNameFormatter();
     
    /**
    * Wether the name is valid
    */
    [Transient]
    private var _nameIsValid : Boolean = true;
     
    /**
    * Wether the name is available
    */
    [Transient]
    private var _nameIsAvailable : Boolean = true;
    
    /**
    * The package compilation errors
    */
    [Transient]
    [ArrayElementType('org.ow2.jasmine.monitoring.eos.rules.common.vo.RulePackageCompilationErrorVO')]
    private var _compilationErrors : ArrayCollection = new ArrayCollection();
    
    /**
    * Wether the compilation has been checked since the last change
    */
    [Transient]
    private var _compilationChecked : Boolean = false;
    
	
    // =========================================================================
    // Getters and setters
    // =========================================================================
    
    // Data values
    
    /**
    * Set the systemName
    */    
    public function set systemName(value:String):void
    {
        // Store the old value
        var oldValue : Object = _systemName;
        
        // Assign the value
    	_systemName = value;
    	
        // Dispatch a property change event
        notifyPropertyChange("systemName",oldValue,value);
    }
    
    /**
    * Get the systemName
    */
    [Bindable]
    public function get systemName():String
    {
    	return _systemName;
    }

    /**
    * Set the previousSystemName
    */
    public function set previousSystemName(value:String):void
    {
        // Store the old value
        var oldValue : Object = _previousSystemName;
        
        // Assign the value
    	_previousSystemName = value;
        
        // Dispatch a property change event
        notifyPropertyChange("previousSystemName",oldValue,value);
    }
    
    /**
    * Get the previousSystemName
    */
    [Bindable]
    public function get previousSystemName():String
    {
    	return _previousSystemName;
    }
    
    /**
    * Set the displayName
    */
    public function set displayName(value:String):void
    {
        // Store the old value
        var oldValue : Object = _displayName;
        
        // Assign the value
    	_displayName = value;
    	
        // Dispatch a property change event
        notifyPropertyChange("displayName",oldValue,value);
        
    	// Update the systemName
    	updateSystemName();
    }
    
    /**
    * Get the displayName
    */
    [Bindable]
    public function get displayName():String
    {
    	return _displayName;
    }
    
    /**
     * Set the name of the rule engine instance on which the rule is executed.
     */
    public function set ruleEngineInstanceName(value:String):void
    {
        // Store the old value
        var oldValue : Object = _ruleEngineInstanceName;
        
        // Assign the value
        _ruleEngineInstanceName = value;
        
        // Dispatch a property change event
        notifyPropertyChange("ruleEngineInstanceName",oldValue,value);
    }
    
    /**
     * Get the name of the rule engine instance on which the rule is executed.
     */
    [Bindable]
    public function get ruleEngineInstanceName():String
    {
        return _ruleEngineInstanceName;
    }
    
    /**
    * Set the state
    */    
    public function set state(value:String):void
    {
        // Store the old value
        var oldValue : Object = _state;
        
        // Assign the value
        _state = value;
        
        // Dispatch a property change event
        notifyPropertyChange("state",oldValue,_state);
    }
    
    /**
    * Get the state
    */    
    [Bindable]
    public function get state():String
    {
    	return _state;
    }
    
    /**
    * Set the description
    */
    public function set description(value:String):void
    {
        // Store the old value
        var oldValue : Object = _description;
        
        // Assign the value
    	_description = value;
        
        // Dispatch a property change event
        notifyPropertyChange("description",oldValue,value);
    }
    
    /**
    * Get the description
    */    
    [Bindable]
    public function get description():String
    {
    	return _description;
    }
    
    
    // Transient properties
    
    /**
    * Set the validity of the name
    */
    public function set nameIsValid(value:Boolean):void
    {
        // Store the old value
        var oldValue : Object = _nameIsValid;
        
        // Assign the value
        _nameIsValid = value;
        
        // Dispatch a property change event
        notifyPropertyChange("nameIsValid",oldValue,value);
    }
    
    /**
    * Check the validity of the name
    */
    [Bindable]
    [Transient]
    public function get nameIsValid():Boolean
    {
        return _nameIsValid;
    }
    
    /**
    * Check the availability of the name
    */
    public function set nameIsAvailable(value:Boolean):void
    {
        // Store the old value
        var oldValue : Object = _nameIsAvailable;
        
        // Assign the value
        _nameIsAvailable = value;
        
        // Dispatch a property change event
        notifyPropertyChange("nameIsAvailable",oldValue,value);
    }
    
    /**
    * Get the availability of the name
    */
    [Bindable]
    [Transient]
    public function get nameIsAvailable():Boolean
    {
        return _nameIsAvailable;
    }
    
    /**
    * Set the compilation errors
    */
    public function set compilationErrors(value:ArrayCollection):void
    {
        // Store the old value
        var oldValue : ArrayCollection = compilationErrors;
        
        // Assign the value
        _compilationErrors = value;
        
        // Dispatch a property change event
        notifyPropertyChange("compilationErrors",oldValue,value);
    }
    
    /**
    * Get the compilation errors
    */
    [Bindable]
    [Transient]
    public function get compilationErrors():ArrayCollection
    {
        return _compilationErrors;
    }

    /**
    * Wether the compilation has been checked since the last change
    */
    public function set compilationChecked(value:Boolean):void
    {
        // Store the old value
        var oldValue : Boolean = compilationChecked;
        
        // Assign the value
        _compilationChecked = value;
        
        // Dispatch a property change event
        notifyPropertyChange("compilationChecked",oldValue,value);
    }
    
    /**
    * Wether the compilation has been checked since the last change
    */
    [Transient]
    public function get compilationChecked():Boolean
    {
        return _compilationChecked;
    }
    
    
    // =========================================================================
    // Public methods
    // =========================================================================
    
    /**
    * Returns a String preview of the object
    */
    override public function toString():String
    {
    	return "{displayName:" + displayName +
    	       ",systemName:" + systemName +
               ",state:" + state + "}";
    }
    
    /**
    * Check if the rule package declares itself as ready to be saved
    */
    public function canBeSaved():String
    {
        var issues : String = "";
        
        // Check that the name is valid and available
        if (!nameIsAvailable) {
            issues += '\nThe name you entered is already taken by another rule package.';
        }
        if (!nameIsValid) {
            issues += '\nThe name you entered is invalid. Special characters are forbidden.';
        }
        
        // If activating/updating the rule package, force:
        //    1. Check compilation
        //    2. Correct the errors
        if (state == ACTIVATED || state == ACTIVATION_REQUESTED) {
            if (!compilationChecked) {
                issues += '\nThe rule package compilation has not been checked since the last ' + 
                          'modification. Check it before you activate it, or save it as deactivated.';
            } else if (compilationErrors.length>0) {
                issues += '\nThe rule package has compilation errors. You cannot activate it. ' + 
                          'Correct the errors or deactivate it before saving it.';
            }
        }
        
        // If issues found, return them
        if (issues.length > 0) {
            return issues;
        }
        
        return null;
    }
    
    /**
    * Get the state to assign to reach the given state
    */
    public function getStateForTarget(value:String):String
    {
        switch(value) {
            case AbstractRulePackageVO.ACTIVATED:
            {
                switch (_state) {
                    case AbstractRulePackageVO.DEACTIVATED:
                    case AbstractRulePackageVO.ACTIVATION_REQUESTED:
                        return AbstractRulePackageVO.ACTIVATION_REQUESTED;
                        break;
                    default:
                        return value;
                        break;
                }
                break;
            }
            case AbstractRulePackageVO.ACTIVATION_REQUESTED:
            {
                switch (_state) {
                    case AbstractRulePackageVO.ACTIVATED:
                    case AbstractRulePackageVO.ACTIVATION_REQUESTED:
                        break;
                    case AbstractRulePackageVO.DEACTIVATION_REQUESTED:
                        return AbstractRulePackageVO.ACTIVATED;
                        break;
                    default:
                        return value;
                        break;
                }
                break;
            }
            case AbstractRulePackageVO.DEACTIVATED:
            {
                switch (_state) {
                    case AbstractRulePackageVO.ACTIVATED:
                    case AbstractRulePackageVO.DEACTIVATION_REQUESTED:
                        return AbstractRulePackageVO.DEACTIVATION_REQUESTED;
                        break;
                    default:
                        return value;
                        break;
                }
                break;
            }
            case AbstractRulePackageVO.DEACTIVATION_REQUESTED:
            {
                switch (_state) {
                    case AbstractRulePackageVO.DEACTIVATED:
                    case AbstractRulePackageVO.DEACTIVATION_REQUESTED:
                        break;
                    case AbstractRulePackageVO.ACTIVATION_REQUESTED:
                        return AbstractRulePackageVO.DEACTIVATED;
                        break;
                    default:
                        return value;
                        break;
                }
                break;
            }
        }
        return value;
    }
    
    
    // =========================================================================
    // Private methods
    // =========================================================================
    
    /**
    * Update the systemName field
    */
    private function updateSystemName(e:Event=null):void
    {
        // Format the systemName from the displayName
        rulepackageNameFormatter.operation = RulePackageNameFormatter.FORMAT_SYSTEM_NAME;
        var formattedRulePackageSystemName:String = 
           rulepackageNameFormatter.format(displayName);
        
        // Check the name validity
        nameIsValid = (formattedRulePackageSystemName != "");
        
        // If needed, store the old identifier
        if (previousSystemName == null) {
            previousSystemName = systemName;
        }
        
        // Assign the new value
        systemName = formattedRulePackageSystemName;
    }
     
}
}
